#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['motr',
 'motr._api',
 'motr._api.actions',
 'motr._api.requirements',
 'motr.controllers',
 'motr.core',
 'motr.ext',
 'motr.plugins',
 'motr.templates']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['attrs',
 'blessed',
 'cement~=3.0, >=3.0.4',
 'colorlog',
 'jinja2',
 'pyrsistent',
 'python-slugify',
 'pyyaml',
 'tqdm',
 'trio']

entry_points = \
{'console_scripts': ['motr = motr.main:main']}

setup(name='motr',
      version='0.1.3',
      description="Max's Obvious Task Runner",
      author='Max Woerner Chase',
      author_email='max.chase@gmail.com',
      url=None,
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.8',
     )
